﻿using System;
using System.Diagnostics;
using System.Text;
using System.Threading;
using KathreinApi2008LanOnly;

namespace Test
{
    public static class Program
    {
        private static string FormatEpc(byte[] epc)
        {
            Debug.Assert(epc.Length == 12);

            var builder = new StringBuilder(12 * 3 - 1);

            builder.AppendFormat("{0:X2}", epc[0]);
            for(int i = 1; i < 12; ++i)
                builder.AppendFormat(":{0:X2}", epc[i]);

            return builder.ToString();
        }

        public static void Main(string[] args)
        {
            var random = new Random();

            var reader = new Reader("192.168.200.200");

            using(reader)
            {
                reader.Connect();
                reader.ReadEpcs += OnReadEpcs;
                reader.WriteEpcNoTagInField += OnWriteEpcNoTagInField;
                reader.WriteEpcNotEnoughEnergy += OnWriteEpcNotEnoughEnergy;
                reader.WriteEpcSuccessful += OnWriteEpcSuccessful;

                while(true)
                {
                    int choice;

                    Console.WriteLine();
                    Console.WriteLine("Select (commands are asynchronous, program automatically waits 800ms):");
                    Console.WriteLine("1) Read EPCs");
                    Console.WriteLine("2) Write random EPC to single tag");
                    Console.WriteLine("3) Exit");

                    try
                    {
                        Console.Write("> ");
                        choice = int.Parse(Console.ReadLine());
                    }
                    catch(FormatException)
                    {
                        Console.WriteLine("Enter a correct number!");
                        continue;
                    }

                    if(choice == 1)
                        reader.ReadEpcsAsync();
                    else if(choice == 2)
                    {
                        var randomEpc = new byte[12];
                        random.NextBytes(randomEpc);
                        reader.WriteEpcAsync(randomEpc);
                    }
                    else if(choice == 3)
                        break;

                    Thread.Sleep(800);

                    Console.WriteLine();
                }
            }
        }

        public static void OnReadEpcs(byte[] epc)
        {
            Debug.Assert(epc.Length == 12);

            Console.WriteLine("Read EPC: {0}", FormatEpc(epc));
        }

        private static void OnWriteEpcNoTagInField()
        {
            Console.WriteLine("No tag in field!");
        }

        private static void OnWriteEpcNotEnoughEnergy()
        {
            Console.WriteLine("Antenna power too low or tag too far away");
        }

        private static void OnWriteEpcSuccessful()
        {
            Console.WriteLine("EPC successfully written to tag");
        }
    }
}
